import {
  width,
  height,
  mapWidth,
  plotWidth,
  margin,
  svg2,
} from './svgGlobals.js';

export function redrawXAxis(
  transitionDuration,
  xScale,
  xAttr,
) {
  svg2.select('#xAxisG').remove();
  svg2.select('#xAxisText').remove();
  svg2
    .append('g')
    .attr('id', 'xAxisG')
    .attr('transform', `translate(0,${height - margin})`) // This controls the vertical position of the Axis
    .transition()
    .duration(transitionDuration)
    .call(d3.axisBottom(xScale));

  svg2
    .append('text')
    .attr('id', 'xAxisText')
    .text(xAttr)
    .attr('x', plotWidth / 2)
    .attr('y', height - margin / 2)
    .style('text-anchor', 'middle');
}

export function redrawYAxis(
  transitionDuration,
  yScale,
  yAttr,
) {
  svg2.select('#yAxisG').remove();
  svg2.select('#yAxisText').remove();

  svg2
    .append('g')
    .attr('id', 'yAxisG')
    .attr('transform', `translate(${margin},0)`) // This controls the vertical position of the Axis
    .transition()
    .duration(transitionDuration)
    .call(d3.axisRight(yScale));

  svg2
    .append('text')
    .attr('id', 'yAxisText')
    .text(yAttr)
    .attr('transform', 'rotate(-90)')
    .attr('x', -height / 2)
    .attr('y', margin / 2);
}

export function toggleAxisScale(
  curScaleType,
  curExtent,
  curAttr,
  axis,
) {
  let pixelRange;
  let cAttr;
  if (axis == 'y') {
    pixelRange = [height - margin, margin];
    cAttr = 'cy';
  } else {
    pixelRange = [margin, plotWidth - margin];
    cAttr = 'cx';
  }

  var axisScale;
  let newScaleType;
  if (curScaleType == 'linear') {
    axisScale = d3
      .scaleLog()
      .domain(curExtent)
      .range(pixelRange);
    newScaleType = 'log';
  } else if (curScaleType == 'log') {
    axisScale = d3
      .scaleLinear()
      .domain(curExtent)
      .range(pixelRange);
    newScaleType = 'linear';
  }

  if (axis == 'y') {
    redrawYAxis(1000, axisScale, curAttr);
  } else {
    redrawXAxis(1000, axisScale, curAttr);
  }

  svg2
    .selectAll('circle.dot')
    .transition()
    .duration(2000)
    .attr(cAttr, (d) => axisScale(d[curAttr]));

  return [newScaleType, axisScale];
}
