// Here's a snippet of code that I ended up using in a couple places,
// so I've refactored it out as a function
//     data: an iterable (typically Array) of all data points
//     attr: the name of the attribute you want the extent of
//     returns: [min, max] of the attr values in data
// Note: min and max only really makes sense for numeric types, so I've
// included a typecast (the "+") - this will keep me from accidentally
// getting tripped up when my attribute is really strings representing
// numbers
export function getDataExtent(data, attr) {
  return d3.extent(
    // For my data, missing values show up as empty strings, which
    // JavaScript then casts to 0. I will filter out all of these to make
    // sure I get the true extent of the non-missing data
    d3.map(data, (r) => +r[attr]).filter((r) => r > 0),
  );
}

function normalizeCountryName(s) {
  return s.replaceAll(' ', '_').toLowerCase();
}

function parseNumStr(s) {
  return +s.replaceAll(',', '').replaceAll('$$', '');
}

export function parseMapData(mapData) {
  mapData.features.forEach((f) => {
    f.properties.name = normalizeCountryName(
      f.properties.name,
    );
  });
  return mapData;
}

export function parseCountryData(csvData, continentData) {
  var countryData = new Map();
  var countryToMapNames = new Map([
    ['United States', 'USA'],
    ['Tanzania', 'United Republic of Tanzania'],
    ['United Kingdom', 'England'],
    ['United Kingdom', 'Wales'],
    ['Republic of Ireland', 'Ireland'],
  ]);
  csvData.forEach((row) => {
    if (countryToMapNames.has(row.Country)) {
      row.Country = countryToMapNames.get(row.Country);
    }
    // Cleaning specific columns of my data
    // Each row is an object with attributes corresponding to the columns,
    // so I am going to overwrite the values for some of the attributes
    // to be the correct type/format
    row.Country_orig = row.Country;
    row.Country = normalizeCountryName(row.Country);
    row.Population_orig = row.Population;
    row.Population = parseNumStr(row.Population);
    row.Urban_population = parseNumStr(
      row.Urban_population,
    );
    row.Urban_rate = row.Urban_population / row.Population;
    row.GDP_per_capita =
      parseNumStr(row.GDP) / row.Population;
    // Since I'm going to be pulling in an extra dataset with Continent data,
    // I'll initialize a default "unknown" value that I will (hopefully!)
    // overwrite with the actual continent name. This is useful since now if
    // I'm missing data for any country, it will have a valid attribute when I
    // select the .Continent, saving me from anoying "undefined" bugs
    row.Continent = 'Unknown';
    countryData.set(row.Country, row);
  });

  continentData.forEach((row) => {
    let name = normalizeCountryName(row.country);
    if (countryData.has(name)) {
      countryData.get(name).Continent =
        normalizeCountryName(row.continent);
    }
  });

  return countryData;
}
